option(null: hard);	

/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström	

	--------------------------------------------------------------------
		swap_fixois create funcs
	--------------------------------------------------------------------		
*/

//--------
// from db
//--------

/*-----------------------------------------------------------------------
  swap_fixois
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(instrument_name 			instrument_name, 
							date option(nullable) 	trade_date 	= null<date>,
							string option(nullable) quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{			
		__instrument c = __instrument(instrument_name,trade_date,quote_side, null<date> );		
		return create_swap_fixois(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}


/*-----------------------------------------------------------------------
  swap_fixois
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{				
		__instrument c = __instrument(instrument_name,trade_date,quote, null<date> );
		return create_swap_fixois(c,error, E_INIT);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}


/*-----------------------------------------------------------------------
  swap_fixois
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(instr_def 					instr_def, 
							date option(nullable) 	trade_date 	= null<date>,
							string option(nullable) quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{				
		__instrument c = __instrument(instr_def,trade_date,quote_side, null<date> );
		return create_swap_fixois(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}
/*-----------------------------------------------------------------------
  swap_fixois
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(instr_def 					instr_def, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{			
		__instrument c = __instrument(instr_def,trade_date,quote, null<date> );
		return create_swap_fixois(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}

//----------------
// from db or not
//----------------

/*-----------------------------------------------------------------------
  func: swap_fixois
  arguments: 
	instr_def: 	can be created either from the db or user defined 				 
				instr_def is copied and modified
				no lookup of quotes
	instrument_name: user defined name

  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(instr_def 				instr_def, 
							string 					instrument_name  ,
							date option(nullable) 	trade_date ,
							date option(nullable) 	settle_date ,							
							date  option(nullable) 	maturity,//maturity has precedence
							string option(nullable) maturity_code,
							number option(nullable)	fix_rate,							
							number 					nominal 	= 100,
							error_info option(nullable) error	= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{
		__instrument c;
		
		if(!null(maturity))
			c = __instrument_fixois_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity,fix_rate,nominal );
		else if(!null(maturity_code))
			c = __instrument_fixois_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity_code,fix_rate,nominal );
		else
			QL_FAIL("invalid maturity/maturity_code");
		
		return create_swap_fixois( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}


//----------------
// no db, swaplib version
// an external swap is created. Note: an external swap has no internal instr_def
// but can be created from an instr_def
//----------------

/*-----------------------------------------------------------------------
  func: swap_fixois
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(ql_fixed_income_swap ql_fixed_income_swap,
						error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{	
		QL_FAIL_COND(CORE_INT.swap_instr_type(ql_fixed_income_swap) != instr_type.SWAP_FIXOIS, "invalid instrument type");

		__instrument c = __instrument_swap(ql_fixed_income_swap);

		return create_swap_fixois( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixois
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois(ql_fix_leg 		ql_fix_leg,
						ql_ois_leg 		ql_ois_leg,
						string 			id,
						error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{			
		
		//QL_FAIL_COND(swap_instr_type(ql_swap_leg1,ql_swap_leg2) != INST_FIX_OIS, "invalid instrument type");

		__instrument c = __instrument_swap(ql_fix_leg,ql_ois_leg,id);

		return create_swap_fixois( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois");
		return null<swap_fixois>;
	}
}
 
module CORE_INT_FIXOIS
{
	/*-----------------------------------------------------------------------
	  ois_df_fwd_helper
	  for single curr "screen"-swap,
	  ----------------------------------------------------------------------*/	
	void ois_df_fwd_helper(logical is_ccy,
						   ..disc_func option(nullable) df_disc_func,
							out fwd_func 				ois_fwd_func,
							disc_func option(nullable) 	ois_fwddf_func,
							ir_index 					ois_ir_index,
						   logical 						endog_df,
						   date option(nullable)		trade_date,
						   date 						settle_date,
						   date option(nullable) 		maturity,
						   string option(nullable) 		maturity_code,
						   out disc_func 				fix_disc_func,
						   out disc_func 				ois_disc_func)
	{
		if(endog_df) {
			if(!null(ois_fwd_func)) {
				date m = null(maturity) ? date_code(maturity_code).apply_fwd(settle_date,ois_ir_index.fixing_calendar()) : maturity;
				if(!null(trade_date))
					ois_disc_func = CORE_INT.on_fwd_to_df(trade_date,m,ois_fwd_func,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),true);
				else
					ois_disc_func = null<disc_func>;
				if(!is_ccy)
					fix_disc_func = ois_disc_func;
				//QL_WARNING(!null(ois_fwddf_func),"");	
			}
			else if(!null(ois_fwddf_func)) {
				
				if(!null(trade_date)){
					ois_disc_func = ois_fwddf_func;
					if(!is_ccy)
						fix_disc_func = ois_fwddf_func;
					ois_fwd_func = fwd_func(ois_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
				}
				else {
					ois_disc_func = null<disc_func>;
					if(!is_ccy)
						fix_disc_func = null<disc_func>;
				}
			}
			else if(!null(df_disc_func)) {
				if(!null(trade_date)){
					ois_disc_func = df_disc_func;
					if(!is_ccy)
						fix_disc_func = df_disc_func;
					ois_fwd_func = fwd_func(ois_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
				}
				else {
					ois_disc_func = null<disc_func>;
					if(!is_ccy)
						fix_disc_func = null<disc_func>;
				}
			}
		}
		else {
			if(!null(df_disc_func)){
				if(!is_ccy)
					fix_disc_func = df_disc_func;
				ois_disc_func = df_disc_func;
			}
			else {
				if(!null(ois_fwd_func)) {
					if(!null(trade_date)){
						date m = null(maturity) ? date_code(maturity_code).apply_fwd(settle_date,ois_ir_index.fixing_calendar()) : maturity;					
						ois_disc_func = CORE_INT.on_fwd_to_df(trade_date,m,ois_fwd_func,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),true);
					}
					else {
						ois_disc_func = null<disc_func>;
					}
					if(!is_ccy)
						fix_disc_func = ois_disc_func;
				}
				else if(!null(ois_fwddf_func)){
					ois_disc_func = ois_fwddf_func;
					if(!is_ccy)
						fix_disc_func = ois_fwddf_func;
				}				
			}

			if(null(ois_fwd_func)) {
				if(!null(trade_date)){
					if(!null(ois_fwddf_func))
						ois_fwd_func = fwd_func(ois_fwddf_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
					else if(!null(ois_disc_func))
						ois_fwd_func = fwd_func(ois_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
				}				
			}
		}
	}
	/*-----------------------------------------------------------------------
	  ois_df_fwd_helper
	  for cross-curr-swap,
	  ----------------------------------------------------------------------*/	
	/*void ois_df_fwd_helper(..disc_func option(nullable) fix_disc_func_in,
						   ..disc_func option(nullable) ois_disc_func_in,
							out fwd_func 				ois_fwd_func,
							disc_func option(nullable) 	ois_fwddf_func,
							ir_index 					ois_ir_index,
						   logical 						endog_df,
						   date 						trade_date,
						   date 						settle_date,
						   date option(nullable) 		maturity,
						   string option(nullable) 		maturity_code,
						   out disc_func 				fix_disc_func,
						   out disc_func 				ois_disc_func)
	{
		if(endog_df) {
			if(!null(ois_fwd_func)) {
				date m = null(maturity) ? date_code(maturity_code).apply_fwd(settle_date,ois_ir_index.fixing_calendar()) : maturity;					
				ois_disc_func = CORE_INT.on_fwd_to_df(trade_date,m,ois_fwd_func,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),true);
				fix_disc_func = ois_disc_func;
				//QL_WARNING(!null(ois_fwddf_func),"");	
			}
			else if(!null(ois_fwddf_func)) {
				ois_disc_func = ois_fwddf_func;
				fix_disc_func = ois_fwddf_func;
				ois_fwd_func = fwd_func(ois_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
			}
			else if(!null(df_disc_func)) {
				ois_disc_func = df_disc_func;
				fix_disc_func = df_disc_func;
				ois_fwd_func = fwd_func(ois_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
			}
		}
		else {
			if(!null(df_disc_func)){
				fix_disc_func = df_disc_func;
				ois_disc_func = df_disc_func;
			}
			else {
				 if(!null(ois_fwd_func)) {
					date m = null(maturity) ? date_code(maturity_code).apply_fwd(settle_date,ois_ir_index.fixing_calendar()) : maturity;					
					ois_disc_func = CORE_INT.on_fwd_to_df(trade_date,m,ois_fwd_func,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),true);
					fix_disc_func = ois_disc_func;
				}
				else if(!null(ois_fwddf_func)){
					ois_disc_func = ois_fwddf_func;
					fix_disc_func = ois_fwddf_func;
				}				
			}

			if(null(ois_fwd_func)) {
				if(!null(ois_fwddf_func))
					ois_fwd_func = fwd_func(ois_fwddf_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
				else if(!null(ois_disc_func))
					ois_fwd_func = fwd_func(ois_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
			}
		}
	}*/
	/*-----------------------------------------------------------------------
	  func: swap_fixois_plain
	  minimal constructor for single curr "screen"-swap,
	  ----------------------------------------------------------------------*/		
	swap_fixois swap_fixois_plain_int(	instr_def 					def,
										string 						instrument_name,	
										date option(nullable) 		trade_date,	
										date option(nullable) 		start_date,
										string option(nullable) 	fwd_start_code,
										date 	option(nullable)	maturity,
										string 	option(nullable)	maturity_code,
										number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
										number 	 					notional,																										
										logical 					pay_fixed,
										..disc_func option(nullable) df_disc_func,								
										fwd_func option(nullable) 	ois_fwd_func,
										disc_func option(nullable) 	ois_fwddf_func,
										number option(nullable) 	curr_fix, //used as proxy for all unknowns
										integer						fix_r_rnd_dec,
										error_info option(nullable) error )
	{	
		try{
			CORE_INT.reset_single(error);
			QL_FAIL_COND(!def.is_swap_fixois(), "instr_def not a fix-ois swap", E_INIT);
			logical is_ccy				= def.is_ccy_swap();
			QL_FAIL_COND(is_ccy, "constructor not applicable for currency swap", E_INIT);
			
			//---def---
			error_info ee = error_info(true,true);
			notional_exchg_style ne = def.swap_not_exchange(ee);

			if(null(ne))
				ne 	= is_ccy ? NE_BOTH : NE_NONE;
			
			date_code settle_code 		= date_code(def.settle_code());		
			coupon_freq  fix_cpn_freq 	= coupon_freq_from_int(def.coupon_freq(ee))  ;
			calendar pmt_calendar		= def.swap_calendar_leg1(ee);
			bd_convention pmt_bus_day 	= def.swap_pmt_bus_day_leg1(ee);
			logical	end_of_month		= def.swap_eom_leg1(ee);

			ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", def.swap_currency_leg1(ee), settle_code, fix_cpn_freq,
														def.swap_dc_method_leg1(ee),pmt_bus_day,	
														pmt_calendar,end_of_month,ne);
			
			
			coupon_freq  ois_cpn_freq 	= coupon_freq_from_int(def.swap_coupon_freq_leg2(ee))  ;
			ir_index ois_ir_index 		= def.swap_ir_index_leg2(ee);

			round_code comp_factor_round_code 		= null<round_code>;
			round_code  imp_rate_pcnt_round_code 	= null<round_code>;										
			interest_rule  ir_rule 					= interest_rule.IR_ADJ; 																
			number  pmt_delay 						= 0;															
			rfr_arrears_type  arr_type 				= rfr_arrears_type.PLAIN;
			number  arr_days 						= 0;
											
			
			ql_swap_ois_def ois_def 	= ql_swap_ois_def("ois_def",def.swap_currency_leg2(ee),settle_code,
															ois_cpn_freq,def.swap_dc_method_leg2(ee),flt_roll_method.FLR_STRAIGHT,pmt_bus_day,
															pmt_calendar,end_of_month,ois_ir_index,null<quote_style>,
															null<round_code>,comp_factor_round_code,imp_rate_pcnt_round_code,
															ir_rule, 	pmt_delay, pmt_delay,pmt_delay,pmt_delay,arr_type,arr_days,
															null<flt_comp_avg_type>,null<flt_sprd_comp_method>,null<flt_avg_method>,
															ne,null<logical>);

			//---leg---
			date  fix_first_cpn_date = null<date>;
			date  fix_last_reg_date = null<date>;
			date  ois_first_cpn_date = null<date>;
			date  ois_last_reg_date = null<date>;
			number	ois_spread 		= 0;		
			integer	cpn_roll_day 	= 0;
			
			number fix_notional		= notional;
			number ois_notional		= notional;
			number fix_fx_rate 		= 1;
			number ois_fx_rate 		= 1;
			vector(date) fixing_dates = null<vector(date)>;	
			vector(number) fixing_rates= null<vector(number)>;
			number fix_proxy 		= curr_fix;
			logical  allow_fwd_fix	= false;
			
			date settle_date;
			CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,pmt_calendar,fwd_start_code,start_date,settle_date);
			QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);
			
			tenor_code mat_code = CORE_INT_FIXFLT.swap_maturity_helper(def,maturity, maturity_code);
			
			if(null(fix_fx_rate)) 
				fix_fx_rate = 1;

			logical endog_df = ois_def.is_endog_df();
			..disc_func fix_disc_func, ois_disc_func;
			ois_df_fwd_helper(is_ccy,df_disc_func,ois_fwd_func,ois_fwddf_func,ois_ir_index,endog_df,trade_date,settle_date,maturity,maturity_code,fix_disc_func,ois_disc_func);
			
			/*if(null(df_disc_func){
				if(!null(ois_fwddf_func)){
					ois_disc_func = ois_fwddf_func;
				}
				else if(!null(ois_fwd_func)) {
					date m = null(maturity) ? date_code(maturity_code).apply_fwd(settle_date,ois_ir_index.fixing_calendar()) : maturity;					
					ois_disc_func = on_fwd_to_df(trade_date,m,ois_fwd_func,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar());
				}
			}
			else {
				
				ois_disc_func = df_disc_func;
				if(null(ois_fwd_func))
					ois_fwd_func = fwd_func(df_disc_func,365,trade_date,ois_ir_index.day_count_method(),ois_ir_index.fixing_calendar(),BD_FOLLOWING,false);
			}
					
			fix_disc_func = ois_disc_func;*/
			
			number init_rate 	= null(fix_coupon_rate) ? 0 : fix_coupon_rate;			
			ql_fix_leg fix 		= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
											pay_fixed,fix_notional,start_date,fix_first_cpn_date,fix_last_reg_date,init_rate,	
											fix_disc_func,cpn_roll_day,fix_fx_rate);

			if(null(ois_notional) && !is_ccy) 
				ois_notional = fix_notional;
			if(null(ois_fx_rate)) 
				ois_fx_rate = 1;

			/*logical allow_extrap = false;
			tenor_surface ois_tenor_surface;
			try{				
				ois_tenor_surface = tenor_surface(trade_date, ois_disc_func,[ois_fwd_func],[ois_ir_index],allow_extrap);
			}
			catch {
				ois_tenor_surface = null<tenor_surface>;
			}*/

			logical ois_approx 	= false;
			ql_ois_leg ois 		= ql_ois_leg(ois_def,"ois_leg",trade_date,settle_date,start_date,maturity,mat_code,
											!pay_fixed,ois_notional,start_date,ois_first_cpn_date,ois_last_reg_date,ois_spread,ois_disc_func,	
											ois_fwd_func,cpn_roll_day,ois_fx_rate,
											fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix, ois_approx);
			
			ql_fixed_income_swap sw = ql_fixed_income_swap(fix,ois,instrument_name);

			if(null(fix_coupon_rate)) {
				number fix_r = sw.solver(0,swap_solver_code.SS_FIX_RATE_LEG1,false);
				QL_FAIL_COND(null(fix_r),"error solving for fixed coupon rate", E_INIT);
				if(fix_r_rnd_dec >= 0)
					fix_r = round(fix_r,fix_r_rnd_dec);
				fix = ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
								pay_fixed,fix_notional,start_date,fix_first_cpn_date,fix_last_reg_date,fix_r,	
								fix_disc_func,cpn_roll_day,fix_fx_rate);
				sw 	= ql_fixed_income_swap(fix,ois,instrument_name);

			}
			return swap_fixois(sw, error);						
		}
		catch {
			CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
			return null<swap_fixois>;
		}
	}

	/*-----------------------------------------------------------------------
	  func: swap_fixois_plain
	  minimal constructor for single curr "screen"-swap
	  this version: based on input of another swap_fixois (separate version because an ext_swap does not have a instr_def)
	  ----------------------------------------------------------------------*/
	swap_fixois swap_fixois_plain_int(	swap_fixois 				sw_in,
										string 						instrument_name,	
										date option(nullable) 		trade_date,	
										date option(nullable) 		start_date,
										string option(nullable) 	fwd_start_code,
										date 	option(nullable)	maturity,
										string 	option(nullable)	maturity_code,
										number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
										number 	 					notional,																										
										logical 					pay_fixed,
										..disc_func option(nullable) df_disc_func,								
										fwd_func option(nullable) 	ois_fwd_func,
										disc_func option(nullable) 	ois_fwddf_func,
										number option(nullable) 	curr_fix,
										integer						fix_r_rnd_dec,
										error_info option(nullable) error )
	{	
		try{
			CORE_INT.reset_single(error);
			
			logical is_ccy				= sw_in.is_ccy_swap();
			QL_FAIL_COND(is_ccy, "constructor not applicable for currency swap", E_INIT);
			
			//---def---
			error_info ee = error_info(true,true);
			//notional_exchg_style ne = def.swap_not_exchange(ee);
			notional_exchg_style ne = notional_exchg_style.NE_NONE;

			//if(null(ne))
				//ne 	= is_ccy ? NE_BOTH : NE_NONE;
			
			date_code settle_code 		= date_code(sw_in.settle_code());
			integer cfreq_fix, cfreq_ois;
			sw_in.coupon_freq(cfreq_fix, cfreq_ois,ee);
			coupon_freq  fix_cpn_freq 	= coupon_freq_from_int(cfreq_fix)  ;
			coupon_freq  ois_cpn_freq 	= coupon_freq_from_int(cfreq_ois)  ;
			
			//calendar pmt_calendar		= def.swap_calendar_leg1(ee);
			//bd_convention pmt_bus_day 	= def.swap_pmt_bus_day_leg1(ee);
			//logical	end_of_month		= def.swap_eom_leg1(ee);
			calendar pmt_calendar_fix, pmt_calendar_ois;
			sw_in.calendar(pmt_calendar_fix,pmt_calendar_ois,ee);
			
			bd_convention pmt_bus_day_fix,pmt_bus_day_ois;
			sw_in.pmt_bus_day(pmt_bus_day_fix,pmt_bus_day_ois,ee);
			
			logical eom_fix,eom_ois;
			sw_in.eom(eom_fix,eom_ois,ee);

			day_count_method dc_fix,dc_ois;
			sw_in.dc_method(dc_fix,dc_ois,ee);

			string ccy_fix,ccy_ois;
			sw_in.currency(ccy_fix,ccy_ois,ee);
			
			ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", ccy_fix, settle_code, fix_cpn_freq,
														dc_fix,pmt_bus_day_fix, pmt_calendar_fix,eom_fix,ne);
			
			
			//coupon_freq  ois_cpn_freq 	= coupon_freq_from_int(def.swap_coupon_freq_leg2(ee))  ;
			ir_index ois_ir_index 					= sw_in.ir_index_ois_leg(ee);
			round_code comp_factor_round_code 		= null<round_code>;
			round_code  imp_rate_pcnt_round_code 	= null<round_code>;										
			interest_rule  ir_rule 					= interest_rule.IR_ADJ; 																
			number  pmt_delay 						= 0;															
			rfr_arrears_type  arr_type 				= rfr_arrears_type.PLAIN;
			number  arr_days 						= 0;
														
			ql_swap_ois_def ois_def 	= ql_swap_ois_def("ois_def",ccy_ois,settle_code,
															ois_cpn_freq,dc_ois,flt_roll_method.FLR_STRAIGHT,pmt_bus_day_ois,
															pmt_calendar_ois,eom_ois,ois_ir_index,null<quote_style>,
															null<round_code>,comp_factor_round_code,imp_rate_pcnt_round_code,
															ir_rule, 	pmt_delay, pmt_delay,pmt_delay,pmt_delay,arr_type,arr_days,
															null<flt_comp_avg_type>,null<flt_sprd_comp_method>,null<flt_avg_method>,
															ne,null<logical>);

			//---leg---
			date  fix_first_cpn_date = null<date>;
			date  fix_last_reg_date = null<date>;
			date  ois_first_cpn_date = null<date>;
			date  ois_last_reg_date = null<date>;
			number	ois_spread 		= 0;		
			integer	cpn_roll_day 	= 0;
			number fix_notional		= notional;
			number ois_notional		= notional;
			number fix_fx_rate 		= 1;
			number ois_fx_rate 		= 1;
			vector(date) fixing_dates = null<vector(date)>;	
			vector(number) fixing_rates= null<vector(number)>;
			number fix_proxy 		= curr_fix;
			logical  allow_fwd_fix	= false;
			
			date settle_date;
			CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,pmt_calendar_fix,fwd_start_code,start_date,settle_date);
			QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);
			
			tenor_code mat_code = CORE_INT_FIXFLT.swap_maturity_helper(null<instr_def>,maturity, maturity_code);
			
			if(null(fix_fx_rate)) 
				fix_fx_rate = 1;

			logical endog_df = ois_def.is_endog_df();
			..disc_func fix_disc_func, ois_disc_func;			
			ois_df_fwd_helper(is_ccy,df_disc_func,ois_fwd_func,ois_fwddf_func,ois_ir_index,endog_df,trade_date,settle_date,maturity,maturity_code,fix_disc_func,ois_disc_func);
			
			ql_fix_leg fix;
			if(null(fix_coupon_rate))
				fix 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
												pay_fixed,fix_notional,start_date,fix_first_cpn_date,fix_last_reg_date,0,	
												fix_disc_func,cpn_roll_day,fix_fx_rate);
			else
				fix 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
												pay_fixed,fix_notional,start_date,fix_first_cpn_date,fix_last_reg_date,fix_coupon_rate,	
												fix_disc_func,cpn_roll_day,fix_fx_rate);

			if(null(ois_notional) && !is_ccy) 
				ois_notional = fix_notional;
			if(null(ois_fx_rate)) 
				ois_fx_rate = 1;

			/*logical allow_extrap = false;
			tenor_surface ois_tenor_surface;
			try{
				ois_tenor_surface = tenor_surface(trade_date, ois_disc_func,[ois_fwd_func],[ois_ir_index],allow_extrap);
			}
			catch {
				ois_tenor_surface = null<tenor_surface>;
			}*/

			logical ois_approx 		= false;
			ql_ois_leg ois 			= ql_ois_leg(ois_def,"ois_leg",trade_date,settle_date,start_date,maturity,mat_code,
												!pay_fixed,ois_notional,start_date,ois_first_cpn_date,ois_last_reg_date,ois_spread,ois_disc_func,	
												ois_fwd_func,cpn_roll_day,ois_fx_rate,
												fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix, ois_approx);
			
			ql_fixed_income_swap sw = ql_fixed_income_swap(fix,ois,instrument_name);

			if(null(fix_coupon_rate)) {
				number fix_r = sw.solver(0,SS_FIX_RATE_LEG1,false);
				QL_FAIL_COND(null(fix_r),"error solving for fixed coupon rate", E_INIT);
				if(fix_r_rnd_dec >= 0)
					fix_r = round(fix_r,fix_r_rnd_dec);
				fix = ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
								pay_fixed,fix_notional,start_date,fix_first_cpn_date,fix_last_reg_date,fix_r,	
								fix_disc_func,cpn_roll_day,fix_fx_rate);
				sw 	= ql_fixed_income_swap(fix,ois,instrument_name);

			}
			return swap_fixois(sw, error);						
		}
		catch {
			CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
			return null<swap_fixois>;
		}
	}
}
/*-----------------------------------------------------------------------
  func: swap_fixois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	instr_def 					def,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,	
								number option(nullable)		fix_coupon_rate ,	//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								fwd_func option(nullable) 	ois_fwd_func,	//	ois_fwd_func has precedence if endog_df
							    number option(nullable) 	curr_fix,
								integer						fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	return CORE_INT_FIXOIS.swap_fixois_plain_int(def,instrument_name,trade_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,df_disc_func,ois_fwd_func, null<disc_func>,
								 curr_fix, fix_r_rnd_dec,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	instr_def 					def,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								disc_func option(nullable) 	ois_fwddf_func,	//	ois_fwddf_func has precedence if endog_df																																		
							    number option(nullable) 	curr_fix,
								integer						fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	return CORE_INT_FIXOIS.swap_fixois_plain_int(def,instrument_name,trade_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,df_disc_func,null<fwd_func>,ois_fwddf_func,
								 curr_fix, fix_r_rnd_dec,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	swap_fixois 				sw,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,	
								number option(nullable)		fix_coupon_rate ,	//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								fwd_func option(nullable) 	ois_fwd_func,		//	ois_fwd_func has precedence if endog_df																																	
							    number option(nullable) 	curr_fix,
							    integer						fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	return CORE_INT_FIXOIS.swap_fixois_plain_int(sw,instrument_name,trade_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,df_disc_func,ois_fwd_func, null<disc_func>,
								 curr_fix, fix_r_rnd_dec,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	swap_fixois 				sw,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								disc_func option(nullable) 	ois_fwddf_func,	//	ois_fwddf_func has precedence if endog_df																																			
							    number option(nullable) 	curr_fix,
								integer						fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	return CORE_INT_FIXOIS.swap_fixois_plain_int(sw,instrument_name,trade_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,df_disc_func,null<fwd_func>, ois_fwddf_func,curr_fix, fix_r_rnd_dec,error );
}


/*-----------------------------------------------------------------------
  func: swap_fixoisplain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	INSTR_TMPL.swap_fixois_def_tmpl	tmpl,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								string option(nullable) 	fwd_start_code,//if tmpl have a code it is ignored
								string 	option(nullable)	maturity_code,//if tmpl have a code it is ignored																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								disc_func option(nullable) 	ois_fwddf_func,		//	ois_fwddf_func has precedence if endog_df																																			
							    number option(nullable) 	curr_fix,
							    integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	try{		
		instr_def def = instr_def_swap_fixois(tmpl,error);		
		if(!CORE_INT.check_instr_def(def))
			return null<swap_fixois>;
		return swap_fixois_plain(	def,instrument_name,trade_date,null<date>,fwd_start_code,
									null<date>,maturity_code,fix_coupon_rate ,
									notional,pay_fixed,df_disc_func,ois_fwddf_func,curr_fix,fix_r_rnd_dec,error);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
		return null<swap_fixois>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixoisplain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	INSTR_TMPL.swap_fixois_def_tmpl	tmpl,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								string option(nullable) 	fwd_start_code,//if tmpl have a code it is ignored
								string 	option(nullable)	maturity_code,//if tmpl have a code it is ignored																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								fwd_func option(nullable) 	ois_fwd_func,	//	ois_fwd_func has precedence if endog_df																																			
							    number option(nullable) 	curr_fix,
								integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	try{		
		instr_def def = instr_def_swap_fixois(tmpl,error);		
		if(!CORE_INT.check_instr_def(def))
			return null<swap_fixois>;
		return swap_fixois_plain(	def,instrument_name,trade_date,null<date>,fwd_start_code,
									null<date>,maturity_code,fix_coupon_rate ,
									notional,pay_fixed,df_disc_func,ois_fwd_func,curr_fix,fix_r_rnd_dec,error);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
		return null<swap_fixois>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixoisplain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	INSTR_TMPL.swap_fixois_def_tmpl	tmpl,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,//if tmpl have a code it is ignored
								date 	option(nullable)	maturity,//if tmpl have a code it is ignored																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								disc_func option(nullable) 	ois_fwddf_func,	//	ois_fwddf_func has precedence if endog_df																																				
							    number option(nullable) 	curr_fix,
								integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	try{		
		instr_def def = instr_def_swap_fixois(tmpl,error);		
		if(!CORE_INT.check_instr_def(def))
			return null<swap_fixois>;
		return swap_fixois_plain(	def,instrument_name,trade_date,start_date,null<string>,
									maturity,null<string>,fix_coupon_rate ,
									notional,pay_fixed,df_disc_func,ois_fwddf_func,curr_fix,fix_r_rnd_dec,error);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
		return null<swap_fixois>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixoisplain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	INSTR_TMPL.swap_fixois_def_tmpl	tmpl,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,//if tmpl have a code it is ignored
								date 	option(nullable)	maturity,//if tmpl have a code it is ignored																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) df_disc_func,								
								fwd_func option(nullable) 	ois_fwd_func,	//	ois_fwd_func has precedence if endog_df																																				
							    number option(nullable) 	curr_fix,
							    integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	try{		
		instr_def def = instr_def_swap_fixois(tmpl,error);		
		if(!CORE_INT.check_instr_def(def))
			return null<swap_fixois>;
		return swap_fixois_plain(	def,instrument_name,trade_date,start_date,null<string>,
									maturity,null<string>,fix_coupon_rate ,
									notional,pay_fixed,df_disc_func,ois_fwd_func,curr_fix,fix_r_rnd_dec,error);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
		return null<swap_fixois>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixois_plain
  this version: using instr_def
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_plain(	instr_def 				instr_def, 
								string 					instrument_name  ,			
								date  option(nullable) 	start_date,
								date  option(nullable)	maturity,
								integer					cpn_roll_day,
								date option(nullable) 	fix_eff_date,
					    		date  option(nullable)	fix_first_cpn_date,
					    		date  option(nullable)	fix_last_reg_date,														
								number  				fix_coupon_rate ,									
								number 	 				fix_notional,
								date option(nullable) 	ois_eff_date,
								date  option(nullable)	ois_first_cpn_date,
					    		date  option(nullable)	ois_last_reg_date,
								number					ois_spread,							
								number 	option(nullable) ois_notional,								
								
								date option(nullable) 		trade_date ,
								date option(nullable) 		settle_date ,
								logical 					pay_fixed,
								disc_func option(nullable) 	fix_disc_func,//note: separate leg disc_funcs even if single currency
								number option(nullable) 	fix_fx_rate,//as of settle
								..disc_func option(nullable) ois_disc_func,	//note: separate leg disc_funcs even if single currency							
								fwd_func option(nullable) 	ois_fwd_func,	//	ois_fwd_func has precedence if endog_df						
								number option(nullable) 	ois_fx_rate,//as of settle						
								vector(date) option(nullable) 	fixing_dates,	
								vector(number) option(nullable) fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 	fix_proxy, 
							    logical  option(nullable) 	allow_fwd_fix,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{	
	try{
		QL_FAIL_COND(!instr_def.is_swap_fixois(), "instr_def not a fix-ois swap", E_INIT);

		//---def---
		logical is_ccy				= instr_def.is_ccy_swap();

		error_info ee 				= error_info(true,true);
		notional_exchg_style ne 	= instr_def.swap_not_exchange(ee);
		
		if(null(ne))
			ne 	= is_ccy ? NE_BOTH : NE_NONE;
		
		date_code settle_code 		= date_code(instr_def.settle_code());		
		coupon_freq  fix_cpn_freq 	= coupon_freq_from_int(instr_def.coupon_freq(ee))  ;
		calendar pmt_calendar		= instr_def.swap_calendar_leg1(ee);
		bd_convention pmt_bus_day 	= instr_def.swap_pmt_bus_day_leg1(ee);
		logical	end_of_month		= instr_def.swap_eom_leg1(ee);

		ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", instr_def.swap_currency_leg1(ee), settle_code, fix_cpn_freq,
													instr_def.swap_dc_method_leg1(ee),pmt_bus_day,	
													pmt_calendar,end_of_month,ne);
		
		//ois_reset_style reset_style = RS_NORM;
		coupon_freq  ois_cpn_freq 	= coupon_freq_from_int(instr_def.swap_coupon_freq_leg2(ee))  ;
		//reset_freq  ois_r_freq 		= reset_freq_from_int(instr_def.swap_reset_freq_leg2(ee))  ;
		ir_index ois_ir_index 		= instr_def.swap_ir_index_leg2(ee);
		flt_comp_avg_type ois_ca_t	= instr_def.swap_comp_avg_type_leg2(ee);
		flt_sprd_comp_method ois_comp_m = instr_def.swap_sprd_comp_method_leg2(ee);
		flt_avg_method ois_avg 		= instr_def.swap_avg_method_leg2(ee);

		round_code comp_factor_round_code 	= null<round_code>;
		round_code imp_rate_pcnt_round_code = null<round_code>;
		number  pmt_delay					= instr_def.swap_pmt_lag_leg2(ee);															
		rfr_arrears_type arr_type 			= instr_def.swap_rfr_arr_type_leg2(ee);
		number arr_days 					= instr_def.swap_rfr_arr_days_leg2(ee);	
		interest_rule ir_rule				= null<interest_rule>;
			
		ql_swap_ois_def ois_def 	= ql_swap_ois_def("ois_def",instr_def.swap_currency_leg2(ee),settle_code,
														ois_cpn_freq,instr_def.swap_dc_method_leg2(ee),pmt_bus_day,
														pmt_calendar,end_of_month,ois_ir_index,comp_factor_round_code,imp_rate_pcnt_round_code,
														ir_rule, pmt_delay,arr_type,arr_days,
														ois_ca_t,ois_comp_m,ois_avg,ne);

		//---leg---
		//date  fix_first_cpn_date = null<date>;
		//date  fix_last_reg_date = null<date>;
		//date  ois_first_cpn_date = null<date>;
		//date  ois_last_reg_date = null<date>;
		CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,pmt_calendar,instr_def.fwd_start_code(ee),start_date,settle_date);
		QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);

		tenor_code mat_code = CORE_INT_FIXFLT.swap_maturity_helper(instr_def, maturity, null<string>);
		
		if(null(fix_fx_rate)) 
			fix_fx_rate = 1;

		logical endog_df = ois_def.is_endog_df();
		string mc = null(maturity) ? instr_def.maturity_code(ee) : null<string>;
		..disc_func df_tmp;	
		CORE_INT_FIXOIS.ois_df_fwd_helper(is_ccy,ois_disc_func,ois_fwd_func,null<disc_func>,
											ois_ir_index,endog_df,trade_date,settle_date,maturity,mc,df_tmp,ois_disc_func);
		
		if(is_ccy){
			QL_REQUIRE(!null(fix_disc_func),"invalid fix leg disc_func", E_INIT);
		}
		else if(null(fix_disc_func)){			
			fix_disc_func = df_tmp;//i.e. allow to be separate even if single curr
		}
		
		
		ql_fix_leg fix = ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
									pay_fixed,fix_notional,fix_eff_date,fix_first_cpn_date,fix_last_reg_date,fix_coupon_rate,	
									fix_disc_func,cpn_roll_day,fix_fx_rate);
		
		if(null(ois_notional) && !is_ccy) 
			ois_notional = fix_notional;
		if(null(ois_fx_rate)) 
			ois_fx_rate = 1;

		/*logical allow_extrap 			= false;
		tenor_surface ois_tenor_surface;
		try {
			ois_tenor_surface = tenor_surface(trade_date, ois_disc_func,[ois_fwd_func],[ois_ir_index],allow_extrap);
		}
		catch {
			ois_tenor_surface = null<tenor_surface>;
		}*/
			
		ql_ois_leg ois 				= ql_ois_leg(ois_def,"float_leg",trade_date,settle_date,start_date,maturity,mat_code,
														!pay_fixed,ois_notional,ois_eff_date,ois_first_cpn_date,ois_last_reg_date,ois_spread,ois_disc_func,	
														ois_fwd_func,cpn_roll_day,ois_fx_rate,
														fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix);
		
		ql_fixed_income_swap sw 		= ql_fixed_income_swap(fix,ois,instrument_name);

		return swap_fixois(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_plain");
		return null<swap_fixois>;
	}
}



module CORE_INT
{
	/*-----------------------------------------------------------------------
	  create_fixois_par
	  ----------------------------------------------------------------------*/
	
	swap_fixois create_fixois_par(instr_class_name 	class_name, 
								string				instrument_name, 									
								date 				trade_date, 
								date				settle_date,
								date 				maturity,
								number				rate)
	
	{
		try{
			error_info ee = new error_info(true,false);
			instr_def i_d 	= instr_def(class_name ,ee);

			ee = new error_info(true,true);
			swap_fixois sw 	= ..swap_fixois(i_d, instrument_name,trade_date,settle_date, 
												maturity,null<string>,rate,100, ee);			
			return sw;
		}
		catch {
			CORE_INT.write_warning_message(err.message());
			return null<swap_fixois>;
		}
	}
}
