option(null: hard);	

/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström

	--------------------------------------------------------------------
		swap_fltflt2s create funcs
	--------------------------------------------------------------------			
*/

//--------
// from db
//--------

/*-----------------------------------------------------------------------
  swap_fltflt2s
  ----------------------------------------------------------------------*/
swap_fltflt2s swap_fltflt2s(instrument_name 			instrument_name, 
								date option(nullable) 		trade_date 	= null<date>,
								string option(nullable) 	quote_side 	= null<string>,
								error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{			
		__instrument c = __instrument(instrument_name,trade_date,quote_side, null<date> );		
		return create_swap_fltflt2s(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		return null<swap_fltflt2s>;
	}
}

/*-----------------------------------------------------------------------
  swap_fltflt2s
  ----------------------------------------------------------------------*/
swap_fltflt2s swap_fltflt2s(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{				
		__instrument c = __instrument(instrument_name,trade_date,quote, null<date> );
		return create_swap_fltflt2s(c,error, E_INIT);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		return null<swap_fltflt2s>;
	}
}


/*-----------------------------------------------------------------------
  swap_fltflt2s
  ----------------------------------------------------------------------*/
swap_fltflt2s swap_fltflt2s(instr_def 					instr_def, 
							date option(nullable) 	trade_date 	= null<date>,
							string option(nullable) quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{				
		__instrument c = __instrument(instr_def,trade_date,quote_side, null<date> );
		return create_swap_fltflt2s(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		return null<swap_fltflt2s>;
	}
}
/*-----------------------------------------------------------------------
  swap_fltflt2s
  ----------------------------------------------------------------------*/
swap_fltflt2s swap_fltflt2s(instr_def 					instr_def, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{			
		__instrument c = __instrument(instr_def,trade_date,quote, null<date> );
		return create_swap_fltflt2s(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		return null<swap_fltflt2s>;
	}
}

//----------------
// from db or not
//----------------


/*-----------------------------------------------------------------------
  func: swap_fltflt2s
  arguments: 
	instr_def: 	can be created either from the db or user defined 				 
				instr_def is copied and modified
				no lookup of quotes
	instrument_name: user defined name

  ----------------------------------------------------------------------*/
swap_fltflt2s swap_fltflt2s(instr_def 				instr_def, 
								string 					instrument_name  ,
								date option(nullable) 	trade_date ,
								date option(nullable) 	settle_date ,							
								date  option(nullable) 	maturity,//maturity has precedence over mat_code
								string option(nullable)	maturity_code,
								number option(nullable)	spread,							
								number 					nominal= 100,
								error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{
		__instrument c;
		if(!null(maturity))
			c = __instrument_fltflt2s_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity,spread,nominal );
		else if(!null(maturity_code))
			c = __instrument_fltflt2s_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity_code,spread,nominal );
		else
			QL_FAIL("invalid maturity/maturity_code");

		return create_swap_fltflt2s( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		return null<swap_fltflt2s>;
	}
}



//----------------
// no db, not par 
//----------------


/*-----------------------------------------------------------------------
  func: swap_fltflt2s
  
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt2s(instr_def 						instr_def, 
							string 							instrument_name  ,			
							date  option(nullable)			maturity,
							integer							cpn_roll_day,
							number 	 						notional,
							number  						fix_spread ,
				   					
							date  							trade_date ,
							date  	option(nullable)		settle_date ,
							logical 						pay_float1,
							fwd_func option(nullable) 		flt1_fwd_func,	
							disc_func option(nullable) 		flt1_disc_func,	
							vector(date) option(nullable) 	flt1_fixing_dates,	
							vector(number) option(nullable) flt1_fixing_rates,
							//number option(nullable) 		first_fixing,
						    //number option(nullable) 		last_fixing,
						    number option(nullable) 		flt1_fix_proxy, 
						    logical  option(nullable) 		flt1_allow_fwd_fix,

							fwd_func option(nullable) 		flt2_fwd_func,	
							disc_func option(nullable) 		flt2_disc_func,																		
							vector(date) option(nullable) 	flt2_fixing_dates,	
							vector(number) option(nullable) flt2_fixing_rates,
							//number option(nullable) 		first_fixing,
						    //number option(nullable) 		last_fixing,
						    number option(nullable) 		flt2_fix_proxy, 
						    logical  option(nullable) 		flt2_allow_fwd_fix,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{
		QL_FAIL_COND(!instr_def.is_swap_fltflt2s(), "instr_def not a float-float swap (2-swap)");

		error_info ee 				= error_info(true,true);
		date_code settle_code 		= date_code(instr_def.settle_code());

		integer freq				= instr_def.coupon_freq(ee);
		coupon_freq  fix_cpn_freq 	= coupon_freq_from_int(freq)  ;		
		calendar pmt_calendar		= instr_def.calendar(0,ee);
		bd_convention pmt_bus_day 	= instr_def.pmt_bus_day(ee);
		logical	end_of_month		= instr_def.eom(ee);

		//-----fix leg------
		ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", instr_def.currency(), settle_code, fix_cpn_freq,
													instr_def.dc_method(ee),pmt_bus_day,pmt_calendar,end_of_month,NE_NONE);

		tenor_code mat_code;
		if(null(maturity)) {
			string mc = instr_def.maturity_code(ee);
			if(!null(mc))
				mat_code = tenor_code(mc);
			else
				QL_FAIL("maturity code not defined in instr_def");
		}
		
		ql_fix_leg fix 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,settle_date,maturity,mat_code,
													false,notional,null<date>,null<date>,null<date>,fix_spread,	
													flt1_disc_func,cpn_roll_day,1);

		number pv_sprd 				= fix.present_value(false);

		//-----float leg defs------
		date  	flt1_first_cpn_date = null<date>;
		date  	flt1_last_reg_date = null<date>;							
		date  	flt2_first_cpn_date = null<date>;
		date  	flt2_last_reg_date = null<date>;

		notional_exchg_style ne = instr_def.swap_not_exchange(ee);
		ne 	= null(ne) ? NE_NONE : ne;	
		
		flt_reset_style reset_style 	= RS_NORM;		
		calendar pmt_calendar_flt		= instr_def.swap_calendar_leg1(ee);
		bd_convention pmt_bus_day_flt 	= instr_def.swap_pmt_bus_day_leg1(ee);
		logical	end_of_month_flt		= instr_def.swap_eom_leg1(ee);

		coupon_freq  flt1_cpn_freq 		= coupon_freq_from_int(instr_def.swap_coupon_freq_leg1(ee))  ;
		reset_freq	flt1_reset_freq;
		if(instr_def.swap_reset_freq_leg1(ee) == err_freq())
			flt1_reset_freq	= null<reset_freq>  ;
		else
			flt1_reset_freq	= reset_freq_from_int(instr_def.swap_reset_freq_leg1(ee))  ;

		flt_comp_avg_type flt1_comp 	= instr_def.swap_comp_avg_type_leg1(ee);
		flt_sprd_comp_method flt1_comp_m = instr_def.swap_sprd_comp_method_leg1(ee);
		flt_avg_method  flt1_avg 		= instr_def.swap_avg_method_leg1(ee);

		ir_index flt1_ir_index 			= instr_def.swap_ir_index_leg1(ee);
				
		ql_swap_float_def flt1_def 		= ql_swap_float_def("float_def1",instr_def.swap_currency_leg1(ee),settle_code,
														flt1_cpn_freq,flt1_reset_freq,instr_def.swap_dc_method_leg1(ee),pmt_bus_day_flt,
														pmt_calendar_flt,end_of_month_flt,
														flt1_comp,flt1_comp_m,flt1_avg,
														flt1_ir_index,reset_style,ne);
				
		coupon_freq  flt2_cpn_freq 		= coupon_freq_from_int(instr_def.swap_coupon_freq_leg2(ee))  ;
		reset_freq	flt2_reset_freq;
		if(instr_def.swap_reset_freq_leg2(ee) == err_freq())
			flt2_reset_freq	= null<reset_freq>  ;
		else
			flt2_reset_freq	= reset_freq_from_int(instr_def.swap_reset_freq_leg2(ee))  ;

		flt_comp_avg_type flt2_comp 	= instr_def.swap_comp_avg_type_leg2(ee);
		flt_sprd_comp_method flt2_comp_m = instr_def.swap_sprd_comp_method_leg2(ee);
		flt_avg_method  flt2_avg 		= instr_def.swap_avg_method_leg2(ee);

		ir_index flt2_ir_index 			= instr_def.swap_ir_index_leg2(ee);
				
		ql_swap_float_def flt2_def 		= ql_swap_float_def("float_def2",instr_def.swap_currency_leg2(ee),settle_code,
														flt2_cpn_freq,flt2_reset_freq,instr_def.swap_dc_method_leg2(ee),pmt_bus_day_flt,
														pmt_calendar_flt,end_of_month_flt,
														flt2_comp,flt2_comp_m,flt2_avg,
														flt2_ir_index,reset_style,ne);

		//-----float legs------

		logical allow_extrap 			= false;

		
		tenor_surface flt1_tenor_surface = tenor_surface(trade_date, flt1_disc_func,[flt1_fwd_func],[flt1_ir_index],allow_extrap);
		tenor_surface flt2_tenor_surface = tenor_surface(trade_date, flt2_disc_func,[flt2_fwd_func],[flt2_ir_index],allow_extrap);
		
		ql_float_leg flt1_flat 			= ql_float_leg(flt1_def,"float_leg1_flat",trade_date,settle_date,settle_date,maturity,mat_code,
														pay_float1,notional,null<date>,flt1_first_cpn_date,flt1_last_reg_date,0,flt1_disc_func,	
														flt1_tenor_surface,cpn_roll_day,1,
														flt1_fixing_dates,flt1_fixing_rates,flt1_fix_proxy,flt1_allow_fwd_fix);		
			
		ql_float_leg flt2_flat 			= ql_float_leg(flt2_def,"float_leg2_flat",trade_date,settle_date,settle_date,maturity,mat_code,
														!pay_float1,notional,null<date>,flt2_first_cpn_date,flt2_last_reg_date,0,flt2_disc_func,	
														flt2_tenor_surface,cpn_roll_day,1,
														flt2_fixing_dates,flt2_fixing_rates,flt2_fix_proxy,flt2_allow_fwd_fix);
	
		ql_fixed_income_swap sw_flat 	= ql_fixed_income_swap(flt1_flat,flt2_flat,"no_name");

		number pv1,pv2;
		number pv = sw_flat.present_value(pv1,pv2,false);
		number pv_adj = pay_float1 ? pv - pv_sprd : pv + pv_sprd;

		number flt1_sprd 				= sw_flat.solver(pv_adj, SS_FLT_SPRD_LEG1,false);

		ql_float_leg flt1 				= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,settle_date,maturity,mat_code,
														pay_float1,notional,null<date>,flt1_first_cpn_date,flt1_last_reg_date,flt1_sprd,flt1_disc_func,	
														flt1_tenor_surface,cpn_roll_day,1,
														flt1_fixing_dates,flt1_fixing_rates,flt1_fix_proxy,flt1_allow_fwd_fix);			

		ql_fixed_income_swap sw 		= ql_fixed_income_swap(flt1,flt2_flat,instrument_name);	

		return swap_fltflt(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		return null<swap_fltflt>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fltflt2s  
  ----------------------------------------------------------------------*/
void swap_fltflt2s(instr_def 						instr_def, 
					string 							instrument_name  ,			
					date  option(nullable)			maturity,
					integer							cpn_roll_day,
					number 	 						notional,
					number  						fix_spread ,
		   					
					date  							trade_date ,
					date  	option(nullable)		settle_date ,
					logical 						pay_fix_swap1,
					fwd_func option(nullable) 		flt1_fwd_func,	
					disc_func option(nullable) 		flt1_disc_func,	
					vector(date) option(nullable) 	flt1_fixing_dates,	
					vector(number) option(nullable) flt1_fixing_rates,
					//number option(nullable) 		first_fixing,
				    //number option(nullable) 		last_fixing,
				    number option(nullable) 		flt1_fix_proxy, 
				    logical  option(nullable) 		flt1_allow_fwd_fix,

					fwd_func option(nullable) 		flt2_fwd_func,	
					disc_func option(nullable) 		flt2_disc_func,																		
					vector(date) option(nullable) 	flt2_fixing_dates,	
					vector(number) option(nullable) flt2_fixing_rates,
					//number option(nullable) 		first_fixing,
				    //number option(nullable) 		last_fixing,
				    number option(nullable) 		flt2_fix_proxy, 
				    logical  option(nullable) 		flt2_allow_fwd_fix,

					out swap_fixflt option(nullable)	fix_flt1,//fix cpn=fix_spread vs flt1 flat
					out swap_fixflt option(nullable)	fix_flt2,//fix cpn = 0 vs flt2 flat
					error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float (2-swap)')
{	
	try{
		QL_FAIL_COND(!instr_def.is_swap_fltflt2s(), "instr_def not a float-float swap (2-swap)");

		date_code settle_code 		= date_code(instr_def.settle_code());
		error_info ee 				= error_info(true,true);
		integer freq				= instr_def.coupon_freq(ee);		
		coupon_freq  fix_cpn_freq 	= coupon_freq_from_int(freq)  ;
		calendar pmt_calendar		= instr_def.calendar(0,ee);
		bd_convention pmt_bus_day 	= instr_def.pmt_bus_day(ee);
		logical	end_of_month		= instr_def.eom(ee);

		//-----fix leg------
		ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", instr_def.currency(), settle_code, fix_cpn_freq,
													instr_def.dc_method(ee),pmt_bus_day,pmt_calendar,end_of_month,NE_NONE);

		tenor_code mat_code;
		if(null(maturity)) {
			string mc = instr_def.maturity_code(ee);
			if(!null(mc))
				mat_code = tenor_code(mc);
			else
				QL_FAIL("maturity code not defined in instr_def");
		}
		
		ql_fix_leg fix1 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,settle_date,maturity,mat_code,
													pay_fix_swap1,notional,null<date>,null<date>,null<date>,fix_spread,	
													flt1_disc_func,cpn_roll_day,1);

		ql_fix_leg fix2 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,settle_date,maturity,mat_code,
													!pay_fix_swap1,notional,null<date>,null<date>,null<date>,0,	
													flt1_disc_func,cpn_roll_day,1);
		
		//-----float leg defs------
		date  	flt1_first_cpn_date = null<date>;
		date  	flt1_last_reg_date = null<date>;							
		date  	flt2_first_cpn_date = null<date>;
		date  	flt2_last_reg_date = null<date>;

		notional_exchg_style ne = instr_def.swap_not_exchange(ee);
		ne 	= null(ne) ? NE_NONE : ne;
				
		flt_reset_style reset_style 	= RS_NORM;		
		calendar pmt_calendar_flt		= instr_def.swap_calendar_leg1(ee);
		bd_convention pmt_bus_day_flt 	= instr_def.swap_pmt_bus_day_leg1(ee);
		logical	end_of_month_flt		= instr_def.swap_eom_leg1(ee);

		coupon_freq  flt1_cpn_freq 		= coupon_freq_from_int(instr_def.swap_coupon_freq_leg1(ee))  ;

		reset_freq	flt1_reset_freq;
		if(instr_def.swap_reset_freq_leg1(ee) == err_freq())
			flt1_reset_freq	= null<reset_freq>  ;
		else
			flt1_reset_freq	= reset_freq_from_int(instr_def.swap_reset_freq_leg1(ee))  ;

		flt_comp_avg_type flt1_comp 	= instr_def.swap_comp_avg_type_leg1(ee);
		flt_sprd_comp_method flt1_comp_m = instr_def.swap_sprd_comp_method_leg1(ee);
		flt_avg_method  flt1_avg 		= instr_def.swap_avg_method_leg1(ee);

		ir_index flt1_ir_index 			= instr_def.swap_ir_index_leg1(ee);
				
		ql_swap_float_def flt1_def 		= ql_swap_float_def("float_def1",instr_def.swap_currency_leg1(ee),settle_code,
														flt1_cpn_freq,flt1_reset_freq,instr_def.swap_dc_method_leg1(ee),pmt_bus_day_flt,
														pmt_calendar_flt,end_of_month_flt,
														flt1_comp,flt1_comp_m,flt1_avg,
														flt1_ir_index,reset_style,ne);
				
		coupon_freq  flt2_cpn_freq 		= coupon_freq_from_int(instr_def.swap_coupon_freq_leg2(ee))  ;
		reset_freq	flt2_reset_freq;
		if(instr_def.swap_reset_freq_leg2(ee) == err_freq())
			flt2_reset_freq	= null<reset_freq>  ;
		else
			flt2_reset_freq	= reset_freq_from_int(instr_def.swap_reset_freq_leg2(ee))  ;

		flt_comp_avg_type flt2_comp 	= instr_def.swap_comp_avg_type_leg2(ee);
		flt_sprd_comp_method flt2_comp_m = instr_def.swap_sprd_comp_method_leg2(ee);
		flt_avg_method  flt2_avg 		= instr_def.swap_avg_method_leg2(ee);

		ir_index flt2_ir_index 			= instr_def.swap_ir_index_leg2(ee);
				
		ql_swap_float_def flt2_def 		= ql_swap_float_def("float_def2",instr_def.swap_currency_leg2(ee),settle_code,
														flt2_cpn_freq,flt2_reset_freq,instr_def.swap_dc_method_leg2(ee),pmt_bus_day_flt,
														pmt_calendar_flt,end_of_month_flt,
														flt2_comp,flt2_comp_m,flt2_avg,
														flt2_ir_index,reset_style,ne);

		//-----float legs------

		logical allow_extrap 			= false;

		tenor_surface flt1_tenor_surface = tenor_surface(trade_date, flt1_disc_func,[flt1_fwd_func],[flt1_ir_index],allow_extrap);
		tenor_surface flt2_tenor_surface = tenor_surface(trade_date, flt2_disc_func,[flt2_fwd_func],[flt2_ir_index],allow_extrap);
		
		ql_float_leg flt1_flat 			= ql_float_leg(flt1_def,"float_leg1_flat",trade_date,settle_date,settle_date,maturity,mat_code,
														!pay_fix_swap1,notional,null<date>,flt1_first_cpn_date,flt1_last_reg_date,0,flt1_disc_func,	
														flt1_tenor_surface,cpn_roll_day,1,
														flt1_fixing_dates,flt1_fixing_rates,flt1_fix_proxy,flt1_allow_fwd_fix);		
			
		ql_float_leg flt2_flat 			= ql_float_leg(flt2_def,"float_leg2_flat",trade_date,settle_date,settle_date,maturity,mat_code,
														pay_fix_swap1,notional,null<date>,flt2_first_cpn_date,flt2_last_reg_date,0,flt2_disc_func,	
														flt2_tenor_surface,cpn_roll_day,1,
														flt2_fixing_dates,flt2_fixing_rates,flt2_fix_proxy,flt2_allow_fwd_fix);
	
		ql_fixed_income_swap sw1 	= ql_fixed_income_swap(fix1,flt1_flat,"fixflt1");
		fix_flt1 = swap_fixflt(sw1, error);

		ql_fixed_income_swap sw2 	= ql_fixed_income_swap(fix2,flt2_flat,"fixflt2");
		fix_flt2 = swap_fixflt(sw2, error);

		return ;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt2s");
		fix_flt1 = null<swap_fixflt>;
		fix_flt2 = null<swap_fixflt>;
		return ;
	}
}

