/*	
	instr_def wrapper functions for swaps
	Developer: Algorithmica Research, Magnus Nyström			
*/

option(null: hard);	


//----------------
// [DATABASE] create
//----------------

/*
instr_def from class: is incomplete
instr_def from instrument: have settle_date/code and maturity_date/code
*/
/*-----------------------------------------------------------------------
  func: instr_def_swap_zeroflt
  ----------------------------------------------------------------------*/
instr_def instr_def_swap_zeroflt(string 	name ,
							error_info option(nullable) error = null<error_info>) 
option (category: 'Instrument/Interest Rate Swap/Zero vs Float')
{	
	try{
		logical	is_class_name;
		..instr_def c = instr_def(name ,is_class_name ,error); 
		c 				= create_instr_def(c, is_class_name, error);		
		QL_FAIL_COND(!null(c) && !c.is_swap_zeroflt(), "instr_def not a zero coupon swap", E_INIT);		
		return c;					
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "instr_def.instr_def_swap_zeroflt");
		return null<..instr_def>;
	}
}



/*-----------------------------------------------------------------------
  func: instr_def_swap_zeroflt [NO DATABASE]  [LEGACY]
  this instr_def will be complete but not date_complete
	arguments:
	instr_def_name: user defined name
  ----------------------------------------------------------------------*/
instr_def instr_def_swap_zeroflt(string				instr_def_name,
								string 				settle_code,
								string option(nullable) maturity_code,
								notional_exchg_style option(nullable) not_exchange,																		
							 	 day_count_method 	zero_day_count_method,
							    string 				zero_currency,
							    calendar 			zero_calendar,
							    bd_convention 		zero_bd_convention,
							    logical 			zero_eom,
								integer 			zero_comp_freq,
							    integer 			flt_cpn_freq,
							    day_count_method 	flt_day_count_method,
							    string 				flt_currency,
							    calendar 			flt_calendar,
							    bd_convention 		flt_bd_convention,
							    logical 			flt_eom,
								// flt_comp_avg_type option(nullable)		flt_comp_avg,
    							//flt_sprd_comp_method option(nullable) 	flt_spread_comp_method,
    							//flt_avg_method option(nullable) 		flt_avg_method,	
								string 				idx_name,
								string 				idx_type,
								string 				idx_tenor,
								integer 			idx_value_days,
							    day_count_method option(nullable) 	idx_day_count_method,
							    bd_convention option(nullable) 		idx_bd_convention,
							    logical option(nullable) 	idx_eom,
							    calendar option(nullable) 	idx_calendar,
							    string option(nullable) 	idx_currency,
							    calendar option(nullable) 	idx_suppl_calendar,							
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Zero vs Float')
{	
	try{	
		if(null(not_exchange)) {
			if( equal_casei(zero_currency, flt_currency)) 
				not_exchange = NE_NONE ;
			else 
				not_exchange = NE_BOTH;
		}

		if(!equal(idx_type, "IBOR") && !equal (idx_type, "LIBOR"))
			QL_FAIL("index type not applicable", E_INIT);
		
		integer idx_spot_days = idx_value_days;
		
		ir_index idx = create_ir_index(	idx_name,idx_type,
										idx_tenor,
										idx_value_days,
										null<number>,
										idx_day_count_method,
										idx_bd_convention,
										idx_eom,
										idx_calendar,//if libor: this is uk
										null<string>,
										idx_currency,
										idx_suppl_calendar,	//if libor: this is primary(ex.target), if ibor: this is null
										null<string>,
										idx_spot_days);
		
		instr_def c = __instr_def_zeroflt_nodb(instr_def_name,zero_day_count_method,zero_currency,zero_calendar,zero_bd_convention,
							    				zero_eom,zero_comp_freq,flt_cpn_freq,flt_day_count_method,flt_currency,flt_calendar,flt_bd_convention,
							    				flt_eom,idx,settle_code,maturity_code,not_exchange );

		return create_instr_def(c, error);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "instr_def.instr_def_swap_zeroflt");
		return null<instr_def>;
	}
}

/*-----------------------------------------------------------------------
  func: instr_def_swap_zeroflt [NO DATABASE]  
  this instr_def will be complete but not date_complete
	arguments:
	instr_def_name: user defined name
  ----------------------------------------------------------------------*/
instr_def instr_def_swap_zeroflt(string				instr_def_name,
								string 				settle_code,
								string option(nullable) maturity_code,
								notional_exchg_style option(nullable) not_exchange,																		
							 	 day_count_method 	zero_day_count_method,
							    string 				zero_currency,
							    calendar 			zero_calendar,
							    bd_convention 		zero_bd_convention,
							    logical 			zero_eom,
								integer 			zero_comp_freq,
							    integer 			flt_cpn_freq,
							    day_count_method 	flt_day_count_method,
							    string 				flt_currency,
							    calendar 			flt_calendar,
							    bd_convention 		flt_bd_convention,
							    logical 			flt_eom,
								// flt_comp_avg_type option(nullable)		flt_comp_avg,
    							//flt_sprd_comp_method option(nullable) 	flt_spread_comp_method,
    							//flt_avg_method option(nullable) 		flt_avg_method,																		    
								ir_index 			flt_idx,							
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Zero vs Float')
{	
	try{	
		if(null(not_exchange)) {
			if( equal_casei(zero_currency, flt_currency)) 
				not_exchange = NE_NONE ;
			else 
				not_exchange = NE_BOTH;
		}
		
		instr_def c = __instr_def_zeroflt_nodb(instr_def_name,zero_day_count_method,zero_currency,zero_calendar,zero_bd_convention,
							    				zero_eom,zero_comp_freq,flt_cpn_freq,flt_day_count_method,flt_currency,flt_calendar,flt_bd_convention,
							    				flt_eom,flt_idx,settle_code,maturity_code,not_exchange );

		return create_instr_def(c, error);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "instr_def.instr_def_swap_zeroflt");
		return null<instr_def>;
	}
}

/*-----------------------------------------------------------------------
  func: instr_def_swap_zeroflt   [NO DATABASE]
  this instr_def will be complete but not date_complete
	arguments:
  	
  ----------------------------------------------------------------------*/

instr_def instr_def_swap_zeroflt(INSTR_TMPL.swap_zeroflt_def_tmpl		tmpl,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Zero vs Float')
{	
	try{
		instr_def c = __instr_def_zeroflt_nodb(	tmpl.name(),tmpl.zero_day_count_method(),tmpl.zero_currency(),tmpl.zero_calendar(),
							    				tmpl.zero_bd_convention(), tmpl.zero_eom(),tmpl.zero_comp_freq(),
												tmpl.flt_cpn_freq(),tmpl.flt_day_count_method(),tmpl.flt_currency(),tmpl.flt_calendar(),
												tmpl.flt_bd_convention(), tmpl.flt_eom(),tmpl.flt_ir_index(),
												tmpl.settle_code(),tmpl.maturity_code(),tmpl.not_exchange() );


		return create_instr_def(c, error);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "instr_def.instr_def_swap_zeroflt");
		return null<instr_def>;
	}
}


/*-----------------------------------------------------------------------
  func: instr_def_swap_zeroflt   [NO DATABASE]
  this instr_def will be complete but not date_complete
	arguments:
  	
  ----------------------------------------------------------------------*/

instr_def instr_def_swap_zeroflt(INSTR_TMPL.swap_zeroflt_def_tmpl		tmpl,
								 string 	option(nullable) 			maturity_code,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Zero vs Float')
{	
	try{
		if(null(maturity_code))
			maturity_code = tmpl.maturity_code();
		instr_def c = __instr_def_zeroflt_nodb(	tmpl.name(),tmpl.zero_day_count_method(),tmpl.zero_currency(),tmpl.zero_calendar(),
							    				tmpl.zero_bd_convention(), tmpl.zero_eom(),tmpl.zero_comp_freq(),
												tmpl.flt_cpn_freq(),tmpl.flt_day_count_method(),tmpl.flt_currency(),tmpl.flt_calendar(),
												tmpl.flt_bd_convention(), tmpl.flt_eom(),tmpl.flt_ir_index(),
												tmpl.settle_code(),maturity_code,tmpl.not_exchange() );


		return create_instr_def(c, error);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "instr_def.instr_def_swap_zeroflt");
		return null<instr_def>;
	}
}
 
